from .document_types import (
    DocumentTypeModelSerializer,
    DocumentTypeRepresentationSerializer,
)
from .documents import DocumentModelSerializer, DocumentRepresentationSerializer, ReadOnlyDocumentModelSerializer
from .shareable_links import (
    ShareableLinkAccessModelSerializer,
    ShareableLinkAccessRepresentationSerializer,
    ShareableLinkModelSerializer,
    ShareableLinkRepresentationSerializer,
)
from .document_model_relationships import DocumentModelRelationshipModelSerializer



# Generated by Django 4.2.7 on 2023-12-06 13:22

from django.db import migrations


class Migration(migrations.Migration):
    atomic = False

    dependencies = [
        ("authentication", "0005_user_external_calendar_settings"),
    ]

    operations = []



# Generated by Django 4.2.6 on 2023-10-24 14:31

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("directory", "0004_entry_is_draft_entry"),
    ]

    operations = [
        migrations.AddField(
            model_name="entry",
            name="salutation",
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name="Salutation"),
        ),
    ]



from django_filters.rest_framework import DjangoFilterBackend as StandardFilterBackend

from .filterset import FilterSet


class DjangoFilterBackend(StandardFilterBackend):
    filterset_base = FilterSet

    def get_filterset_kwargs(self, request, queryset, view):
        return {"data": request.query_params, "queryset": queryset, "request": request, "view": view}

    def get_filterset_class(self, view, queryset=None):
        if method := getattr(view, "get_filterset_class", None):
            view.filterset_class = method(view.request)

        if method := getattr(view, "get_filterset_fields", None):
            view.filterset_fields = method(view.request)

        return super().get_filterset_class(view, queryset)



from wbcore import serializers as wb_serializers

from .models import Geography


class GeographyRepresentationSerializer(wb_serializers.RepresentationSerializer):
    class Meta:
        model = Geography
        fields = ["id", "representation"]


class CountryRepresentationSerializer(GeographyRepresentationSerializer):
    filter_params = {"level": Geography.Level.COUNTRY.value}


class GeographyModelSerializer(wb_serializers.ModelSerializer):
    _parent = GeographyRepresentationSerializer(source="parent")

    class Meta:
        model = Geography
        fields = [
            "id",
            "name",
            "representation",
            "parent",
            "_parent",
            "code_2",
            "code_3",
        ]



from rest_framework.reverse import reverse

from wbcore.metadata.configs.endpoints import EndpointViewConfig


class CurrencyFXRatesCurrencyEndpointConfig(EndpointViewConfig):
    def get_endpoint(self, **kwargs):
        return None

    def get_list_endpoint(self, **kwargs):
        return reverse(
            "wbcore:currency:currency-currencyfxrates-list",
            args=[self.view.kwargs["currency_id"]],
            request=self.request,
        )



import pytest
from django.forms.models import model_to_dict
from rest_framework.exceptions import ValidationError
from rest_framework.test import APITestCase
from wbcore.contrib.example_app.factories import (
    LeagueFactory,
    TeamFactory,
    TeamResultsFactory,
)
from wbcore.contrib.example_app.serializers import TeamResultsModelSerializer


@pytest.mark.django_db
class TestTeamResultSerializer(APITestCase):
    def test_result_serializer(self):
        team = TeamFactory()
        league = LeagueFactory()
        result: dict = model_to_dict(TeamResultsFactory.build(team=team, league=league))
        result["games_played"] = 3
        result_serializer = TeamResultsModelSerializer(data=result)

        self.assertTrue(result_serializer.is_valid())

    def test_same_result(self):
        result = TeamResultsFactory()
        new_result: dict = model_to_dict(TeamResultsFactory.build(team=result.team, league=result.league))
        new_result["games_played"] = 3

        with pytest.raises(ValidationError):
            TeamResultsModelSerializer(data=new_result).is_valid(raise_exception=True)



from .endpoints import (
    PlayerTeamEndpointConfig,
    MatchStadiumEndpointConfig,
    EventMatchEndpointConfig,
    TeamStadiumEndpointConfig,
    EventTypeSportEndpointConfig,
    EventEndpointConfig,
    MatchLeagueEndpointConfig,
    LeagueSportEndpointConfig,
    TeamResultsEndpointConfig,
    LeagueStatisticsEndpointConfig,
)



from django.contrib.auth import get_user_model
from django.db import models
from django.utils.translation import gettext_lazy as _

from wbcore.release_notes.utils import parse_release_note


class ReleaseNote(models.Model):
    """
    This model holds all relevant information about a new release. Instances of this model will never be created
    manually, but always through a management command that manages them from markdown files. Those markdown files
    sit in a dedicated folder called `release_notes`. There are two exceptions for this:
    - The release notes of wbcore: `wbcore/release_notes/release_notes`
    - The release notes of the frontend: `wbcore/release_notes/frontend_release_notes`

    The markdown files have to abide by a certain format, which includes metadata from which all fields are filled.
    The markdown content is parsed through python-markdown and is saved as html. With every deployment, the
    management command `handle_release_notes` is called.
    """

    version = models.CharField(max_length=18, help_text=_("The version/identifier of the release"))
    release_date = models.DateField(help_text=_("The date when this new version was released"))
    module = models.CharField(max_length=255, help_text=_("The workbench module of the release"))
    summary = models.CharField(max_length=512, help_text=_("A brief summary of the release"))
    notes = models.TextField(default="", help_text=_("What's new? What's improved? What's fixed?"))
    read_by = models.ManyToManyField(
        to=get_user_model(), related_name="read_patch_notes", db_table="bridger_releasenote_read_by"
    )

    @classmethod
    def handle_from_markdown(cls, markdown_content: str) -> "ReleaseNote":
        content, meta, version, module, summary, release_date = parse_release_note(markdown_content)

        try:
            release_note = cls.objects.get(version=version, module=module)
            release_note.summary = summary
            release_note.release_date = release_date
            release_note.notes = content
            release_note.save()
        except cls.DoesNotExist:
            release_note = cls.objects.create(
                version=version,
                module=module,
                release_date=release_date,
                summary=summary,
                notes=content,
            )

        return release_note

    def __str__(self) -> str:
        return f"{self.module}: {self.version}"

    @classmethod
    def get_endpoint_basename(cls):
        return "wbcore:releasenote"

    class Meta:
        verbose_name = _("Release Note")
        verbose_name_plural = _("Release Notes")

        constraints = [
            models.UniqueConstraint(fields=["version", "module"], name="release_note_unique_version_module")
        ]

        db_table = "bridger_releasenote"



from django.dispatch import receiver

from wbcore.test.signals import custom_update_kwargs, get_custom_factory

from ..factories import UserFactory
from ..viewsets import UserActivityTable, UserActivityUserChart


@receiver(get_custom_factory, sender=UserActivityTable)
def receive_factory_user_activity(sender, *args, **kwargs):
    return UserFactory


@receiver(custom_update_kwargs, sender=UserActivityUserChart)
def receive_kwargs_user_activity(sender, *args, **kwargs):
    if obj := kwargs.get("obj_factory"):
        return {"user_id": obj.user.id}
    return {}



import inspect
import re
from collections import defaultdict

from django.db.models.signals import *


class DisableSignals(object):
    def __init__(self, disabled_signals=None):
        self.stashed_signals = defaultdict(list)
        self.disabled_signals = disabled_signals or [
            pre_init,
            post_init,
            pre_save,
            post_save,
            pre_delete,
            post_delete,
            pre_migrate,
            post_migrate,
        ]

    def __enter__(self):
        for signal in self.disabled_signals:
            self.disconnect(signal)

    def __exit__(self, exc_type, exc_val, exc_tb):
        for signal in list(self.stashed_signals):
            self.reconnect(signal)

    def disconnect(self, signal):
        self.stashed_signals[signal] = signal.receivers
        signal.receivers = []

    def reconnect(self, signal):
        signal.receivers = self.stashed_signals.get(signal, [])
        del self.stashed_signals[signal]


# https://stackoverflow.com/questions/55578230/django-how-to-visualize-signals-and-save-overrides
RECEIVER_MODELS = re.compile(r"sender=(\w+)\W")


class DisableSignalsNotification(DisableSignals):
    def __enter__(self):
        for signal in self.disabled_signals:
            if not isinstance(signal, ModelSignal):
                continue
            for _, receiver in signal.receivers:
                rcode = inspect.getsource(receiver())
                rmodel = RECEIVER_MODELS.findall(rcode)
                if "Notification" in rmodel:
                    self.disconnect(signal)



from typing import Any

from django.contrib import admin
from dynamic_preferences.registries import global_preferences_registry

from wbcore.contrib.io.admin import ProviderGenericInline

from .models import Currency, CurrencyFXRates


@admin.register(Currency)
class CurrencyModelAdmin(admin.ModelAdmin):
    list_display = (
        "key",
        "title",
    )
    search_fields = ("key", "title", "symbol")
    raw_id_fields = ["import_source"]

    def refetch_import_data(self: Any, request: Any, queryset: Any) -> Any:
        default_start_date = global_preferences_registry.manager()["currency__default_start_date_historical_import"]
        CurrencyFXRates.import_data(start=default_start_date, queryset=queryset)

    actions = [refetch_import_data]
    inlines = [ProviderGenericInline]


@admin.register(CurrencyFXRates)
class CurrencyFXRatesModelAdmin(admin.ModelAdmin):
    list_display = ("currency", "date", "value")
    search_fields = ("currency__key", "currency__title", "currency__symbol")
    list_filter = ["currency"]
    raw_id_fields = ["import_source"]



from django.contrib import admin
from django.db.models import Q
from django.forms import ModelForm
from reversion.admin import VersionAdmin

from ..models import (
    ClientManagerRelationship,
    EmployerEmployeeRelationship,
    Position,
    Relationship,
    RelationshipType,
)


@admin.register(Position)
class PositionAdmin(admin.ModelAdmin):
    search_fields = ("title",)
    list_display = ("id", "title")


@admin.register(Relationship)
class RelationshipModelAdmin(VersionAdmin):
    list_display = ("id", "relationship_type", "from_entry", "to_entry")
    autocomplete_fields = ("relationship_type",)

    def reversion_register(self, model, **options):
        options = {
            "follow": (
                "from_entry",
                "to_entry",
                "relationship_type",
            )
        }
        super().reversion_register(model, **options)


class RelationshipTypeModelAdminForm(ModelForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["counter_relationship"].queryset = RelationshipType.objects.filter(
            ~Q(id=self.instance.id) & (Q(counter_relationship=self.instance) | Q(counter_relationship=None))
        )


@admin.register(RelationshipType)
class RelationshipTypeModelAdmin(VersionAdmin):
    form = RelationshipTypeModelAdminForm
    list_display = ("title", "counter_relationship")
    search_fields = ("title",)

    def reversion_register(self, model, **options):
        options = {"follow": ("counter_relationship",)}
        super().reversion_register(model, **options)


@admin.register(ClientManagerRelationship)
class ClientManagerRelationshipModelAdmin(VersionAdmin):
    search_fields = ("relationship_manager__computed_str", "client__computed_str")
    list_display = ("id", "relationship_manager", "client", "primary", "status", "created")

    def reversion_register(self, model, **options):
        options = {
            "follow": (
                "client",
                "relationship_manager",
            )
        }
        super().reversion_register(model, **options)


@admin.register(EmployerEmployeeRelationship)
class EmployerEmployeeRelationshipAdmin(admin.ModelAdmin):
    list_display = ("id", "employer", "employee", "primary", "position", "position_name")



import pytest
from django.conf import settings
from django.template import TemplateSyntaxError
from pytest_mock import MockerFixture
from wbcore.contrib.workflow.models import EmailStep, ProcessStep


class TestEmailStep:
    @pytest.fixture
    def mocked_email_step(self, mocker: MockerFixture):
        mocked_email_step = mocker.MagicMock(spec=EmailStep)
        mocked_email_step.subject = "Test Subject"
        mocked_email_step.template.file = "<html>Email content</html>"
        mocked_email_step.to.values_list.return_value = ["to@example.com"]
        mocked_email_step.cc.values_list.return_value = [
            "cc1@example.com",
            "cc2@example.com",
        ]
        mocked_email_step.bcc.values_list.return_value = []
        return mocked_email_step

    @pytest.fixture
    def mocked_process_step(self, mocker: MockerFixture):
        mocked_process_step = mocker.MagicMock(spec=ProcessStep)
        mocked_process_step.get_endpoint_basename.return_value = "process-step"
        mocked_process_step.id = 123
        return mocked_process_step

    @pytest.fixture(autouse=True)
    def patching(self, mocker: MockerFixture):
        mocker.patch(
            "wbcore.contrib.workflow.models.step.base_domain",
            return_value="https://testserver.com",
        )
        mocker.patch(
            "wbcore.contrib.workflow.models.step.reverse",
            return_value="/api/process-step/123/",
        )

    def test_run(self, mocker: MockerFixture, mocked_process_step, mocked_email_step):
        # Arrange
        mocked_template_render = mocker.patch(
            "django.template.Template.render",
            return_value="<html>Rendered content</html>",
        )
        mocked_email_init = mocker.patch("django.core.mail.EmailMultiAlternatives.__init__", return_value=None)
        mocked_email_send = mocker.patch("django.core.mail.EmailMultiAlternatives.send")
        mocked_attach_alternative = mocker.patch("django.core.mail.EmailMultiAlternatives.attach_alternative")
        mocker.patch(
            "wbcore.contrib.workflow.models.step.convert_html2text",
            return_value="Rendered content",
        )
        mocked_execute_single_next_step = mocker.patch.object(mocked_email_step, "execute_single_next_step")
        # Act
        EmailStep.run(mocked_email_step, mocked_process_step)
        # Assert
        mocked_template_render.assert_called_once_with(mocker.ANY)
        mocked_email_init.assert_called_once_with(
            "Test Subject",
            "Rendered content",
            settings.DEFAULT_FROM_EMAIL,
            to=["to@example.com"],
            cc=["cc1@example.com", "cc2@example.com"],
            bcc=[],
        )
        mocked_attach_alternative.assert_called_once_with("<html>Rendered content</html>", "text/html")
        mocked_email_send.assert_called_once()
        mocked_execute_single_next_step.assert_called_once_with(mocked_process_step)

    def test_run_failed(self, mocker: MockerFixture, mocked_process_step, mocked_email_step):
        # Arrange
        mocked_template_render = mocker.patch(
            "django.template.Template.render",
            side_effect=TemplateSyntaxError("Invalid syntax"),
        )
        mocked_email_send = mocker.patch("django.core.mail.EmailMultiAlternatives.send")
        mocked_execute_single_next_step = mocker.patch.object(mocked_email_step, "execute_single_next_step")
        mocked_set_failed = mocker.patch.object(mocked_email_step, "set_failed")
        # Act
        EmailStep.run(mocked_email_step, mocked_process_step)
        # Assert
        mocked_template_render.assert_called_once_with(mocker.ANY)
        mocked_email_send.assert_not_called()
        mocked_execute_single_next_step.assert_not_called()
        mocked_set_failed.assert_called_once_with(mocked_process_step, "Error in template syntax!")



from ..models import DataBackend, Provider


def register(
    backend_title: str,
    save_data_in_import_source: bool | None = False,
    passive_only: bool | None = False,
    provider_key: str | None = None,
):
    """
    Decorator to include when a backend need automatic registration
    Args:
        backend_name:

    Returns:

    """
    if not backend_title:
        raise ValueError("At least one name must be passed to register.")

    def _decorator(backend_class):
        provider = None
        if provider_key:
            provider, created = Provider.objects.get_or_create(
                key=provider_key, defaults={"title": provider_key.capitalize()}
            )
        DataBackend.objects.update_or_create(
            backend_class_path=backend_class.__module__,
            backend_class_name=backend_class.__name__,
            defaults={
                "title": f"{backend_title} ({provider.title})" if provider else backend_title,
                "save_data_in_import_source": save_data_in_import_source,
                "passive_only": passive_only,
                "provider": provider,
            },
        )
        return backend_class

    return _decorator



import re
from datetime import datetime
from io import BytesIO, StringIO
from typing import Generator, Optional

from anymail.inbound import AnymailInboundMessage
from django.db.models import Model
from slugify import slugify
from wbcore.contrib.io.backends.abstract import AbstractDataBackend
from wbcore.contrib.io.backends.utils import register


#### IMPORTANT: If this class path change, it needs to be adapted also on the io.backends.handle_inbound function
@register("Default Mail", save_data_in_import_source=True, passive_only=True)
class DataBackend(AbstractDataBackend):
    def get_files(
        self,
        execution_time: datetime,
        file_name_regex: Optional[str] = None,
        message: Optional[AnymailInboundMessage] = None,
        import_credential: Optional[Model] = None,
        import_email_as_file: Optional[bool] = False,
        **kwargs,
    ) -> Generator[tuple[str, BytesIO], None, None]:
        """

        Args:
            execution_time: The time at which this import was called
            file_name_regex: The regex applied on the imported file for validation. Defaults to False.
            message: The AnymailInboundMessage received from the anymail inbounc
            import_credential: Import credential attached to the calling source. Defaults to None.
            import_email_as_file: If false, import the attachment as import source file. Otherwise, import the whole email as file
            **kwargs:

        Returns:

        """
        if message:
            if import_email_as_file:
                filename = f"{slugify(message.subject)}_{datetime.timestamp(execution_time)}.eml"
                yield filename, StringIO(message.as_string())
            elif file_name_regex:
                attachments = message.attachments
                attachments.extend(message.inline_attachments.values())

                for attachment in attachments:
                    f = attachment.as_uploaded_file()
                    f_name = attachment.get_filename()
                    result = re.findall(file_name_regex, f_name)
                    if len(result) > 0:
                        yield f_name, f.file



from unittest.mock import patch

import pytest
from pytest import FixtureRequest
from rest_framework import status

from wbcore.contrib.authentication.factories import (
    GroupFactory,
    UserFactory,
)
from wbcore.contrib.workflow.models import (
    Condition,
    Data,
    DecisionStep,
    Process,
    ProcessStep,
    Transition,
    Workflow,
)
from wbcore.contrib.workflow.viewsets import (
    AssignedProcessStepModelViewSet,
    ConditionModelViewSet,
    DataModelViewSet,
    DecisionStepModelViewSet,
    ProcessModelViewSet,
    ProcessStepModelViewSet,
    TransitionModelViewSet,
    WorkflowModelViewSet,
)


@pytest.mark.django_db
class TestViewSets:
    @pytest.mark.parametrize(
        "viewset, entries",
        [
            (AssignedProcessStepModelViewSet, "assigned_process_steps"),
            (ConditionModelViewSet, "conditions"),
            (DataModelViewSet, "data"),
            (DecisionStepModelViewSet, "decision_steps"),
            (ProcessModelViewSet, "processes"),
            (ProcessStepModelViewSet, "process_steps"),
            (TransitionModelViewSet, "transitions"),
            (WorkflowModelViewSet, "workflows"),
        ],
    )
    def test_get(self, request_factory, super_user, viewset, entries, request: FixtureRequest):
        # Arrange
        entries = request.getfixturevalue(entries)
        request = request_factory.get("")
        request.user = super_user
        viewset = viewset.as_view({"get": "list"})
        # Act
        response = viewset(request)
        # Assert
        assert response.data["count"] == 3
        assert response.status_code == status.HTTP_200_OK

    @pytest.mark.parametrize(
        "viewset, model",
        [
            (AssignedProcessStepModelViewSet, "assigned_process_step"),
            (ConditionModelViewSet, "condition"),
            (DataModelViewSet, "singular_data"),
            (DecisionStepModelViewSet, "decision_step"),
            (ProcessModelViewSet, "process"),
            (ProcessStepModelViewSet, "process_step"),
            (TransitionModelViewSet, "transition"),
            (WorkflowModelViewSet, "workflow"),
        ],
    )
    def test_retrieve(self, request_factory, super_user, viewset, model, request: FixtureRequest):
        # Arrange
        entries = request.getfixturevalue(model)
        request = request_factory.get("")
        request.user = super_user
        vs = viewset.as_view({"get": "retrieve"})
        # Act
        response = vs(request, pk=entries.id)
        instance = response.data.get("instance")
        # Assert
        assert instance is not None
        assert instance["id"] == entries.id
        assert response.status_code == status.HTTP_200_OK

    @pytest.mark.parametrize(
        "viewset, build_data, can_create",
        [
            (AssignedProcessStepModelViewSet, "assigned_process_steps_build", False),
            (ConditionModelViewSet, "condition_build", True),
            (DataModelViewSet, "data_build", True),
            (DecisionStepModelViewSet, "decision_step_build", True),
            (ProcessModelViewSet, "process_build", False),
            (ProcessStepModelViewSet, "process_steps_build", False),
            (TransitionModelViewSet, "transition_build", True),
            (WorkflowModelViewSet, "workflow_build", True),
        ],
    )
    def test_create(
        self,
        request_factory,
        super_user,
        viewset,
        build_data,
        can_create,
        request: FixtureRequest,
    ):
        # Arrange
        data = request.getfixturevalue(build_data)
        request = request_factory.post("", data=data, format="json")
        request.user = super_user
        vs = viewset.as_view({"post": "create"})
        # Act
        response = vs(request)
        expected_status_code = status.HTTP_201_CREATED if can_create else status.HTTP_405_METHOD_NOT_ALLOWED
        # Assert
        assert response.status_code == expected_status_code

    @pytest.mark.parametrize(
        "viewset, entries, model",
        [
            (AssignedProcessStepModelViewSet, "assigned_process_steps", ProcessStep),
            (ConditionModelViewSet, "conditions", Condition),
            (DataModelViewSet, "data", Data),
            (DecisionStepModelViewSet, "decision_steps", DecisionStep),
            (ProcessModelViewSet, "processes", Process),
            (ProcessStepModelViewSet, "process_steps", ProcessStep),
            (TransitionModelViewSet, "transitions", Transition),
            (WorkflowModelViewSet, "workflows", Workflow),
        ],
    )
    def test_delete(
        self,
        request_factory,
        super_user,
        viewset,
        entries,
        model,
        request: FixtureRequest,
    ):
        # Arrange
        entries = request.getfixturevalue(entries)
        entry_id = entries[1].id
        request = request_factory.delete("", args=entry_id)
        request.user = super_user
        vs = viewset.as_view({"delete": "destroy"})
        # Act
        response = vs(request, pk=entry_id)
        # Assert
        assert response.status_code == status.HTTP_204_NO_CONTENT
        assert model.objects.count() == 2
        assert not model.objects.filter(id=entry_id).exists()

    @pytest.mark.parametrize(
        "viewset, entry, new_entry",
        [
            (
                AssignedProcessStepModelViewSet,
                "assigned_process_step",
                "assigned_process_steps_build",
            ),
            (ConditionModelViewSet, "condition", "condition_build"),
            (DataModelViewSet, "singular_data", "data_build"),
            (DecisionStepModelViewSet, "decision_step", "decision_step_build"),
            (ProcessModelViewSet, "process", "process_build"),
            (ProcessStepModelViewSet, "process_step", "process_steps_build"),
            (TransitionModelViewSet, "transition", "transition_build"),
            (WorkflowModelViewSet, "workflow", "workflow_build"),
        ],
    )
    def test_put(
        self,
        request_factory,
        super_user,
        viewset,
        entry,
        new_entry,
        request: FixtureRequest,
    ):
        # Arrange
        entry = request.getfixturevalue(entry)
        new_data = request.getfixturevalue(new_entry)
        new_data["id"] = entry.id
        request = request_factory.put("", data=new_data, format="json")
        request.user = super_user
        vs = viewset.as_view({"put": "update"})
        # Act
        response = vs(request, pk=entry.id)
        # Assert
        assert response.status_code == status.HTTP_200_OK

    @pytest.mark.parametrize(
        "viewset, entry, field, is_editable",
        [
            (AssignedProcessStepModelViewSet, "assigned_process_step", "status", False),
            (ConditionModelViewSet, "condition", "expected_value", True),
            (DataModelViewSet, "singular_data", "label", True),
            (DecisionStepModelViewSet, "decision_step", "status", True),
            (ProcessModelViewSet, "process", "state", False),
            (ProcessStepModelViewSet, "process_step", "status", False),
            (TransitionModelViewSet, "transition", "name", True),
            (WorkflowModelViewSet, "workflow", "name", True),
        ],
    )
    def test_patch(
        self,
        request_factory,
        super_user,
        viewset,
        entry,
        field,
        is_editable,
        request: FixtureRequest,
    ):
        # Arrange
        entry = request.getfixturevalue(entry)
        old_field_data = getattr(entry, field)
        new_field_data = "Foo Bar"
        request = request_factory.patch("", data={field: new_field_data})
        request.user = super_user
        vs = viewset.as_view({"patch": "partial_update"})
        # Act
        response = vs(request, pk=entry.id)
        field_data = new_field_data if is_editable else old_field_data
        entry.refresh_from_db()
        # Assert
        assert response.status_code == status.HTTP_200_OK
        assert getattr(entry, field) == field_data

    def test_get_process_step_su(self, request_factory, super_user, permission, process_step_factory):
        # Arrange
        process_step_a = process_step_factory(permission=permission)
        process_step_b = process_step_factory()
        request = request_factory.get("")
        request.user = super_user
        # Act
        vs = ProcessStepModelViewSet(request=request, kwargs={})
        qs = vs.get_queryset()
        id_list = list(map(str, qs.values_list("id", flat=True)))
        # Assert
        assert process_step_a.id in id_list
        assert process_step_b.id in id_list

    def test_get_queryset_user_permission(self, request_factory, user, permission, process_step_factory):
        # Arrange
        user.user_permissions.add(permission)
        process_step_a = process_step_factory(permission=permission)
        process_step_b = process_step_factory()
        request = request_factory.get("")
        request.user = user
        # Act
        vs = ProcessStepModelViewSet(request=request, kwargs={})
        qs = vs.get_queryset()
        id_list = list(map(str, qs.values_list("id", flat=True)))
        # Assert
        assert process_step_a.id in id_list
        assert process_step_b.id in id_list

    def test_get_queryset_user_no_permission(self, request_factory, user, permission, process_step_factory):
        # Arrange
        process_step_a = process_step_factory(permission=permission)
        process_step_b = process_step_factory()
        request = request_factory.get("")
        request.user = user
        # Act
        vs = ProcessStepModelViewSet(request=request, kwargs={})
        qs = vs.get_queryset()
        id_list = list(map(str, qs.values_list("id", flat=True)))
        # Assert
        assert process_step_a.id not in id_list
        assert process_step_b.id in id_list

    def test_get_queryset_group_permission(self, request_factory, permission, process_step_factory):
        # Arrange
        process_step_a = process_step_factory(permission=permission)
        process_step_b = process_step_factory()
        group = GroupFactory(permissions=[permission])
        user = UserFactory(is_superuser=False, groups=[group])
        request = request_factory.get("")
        request.user = user
        # Act
        vs = ProcessStepModelViewSet(request=request, kwargs={})
        qs = vs.get_queryset()
        id_list = list(map(str, qs.values_list("id", flat=True)))
        # Assert
        assert process_step_a.id in id_list
        assert process_step_b.id in id_list

    @patch("wbcore.contrib.workflow.models.step.Step.start_next_step")
    def test_next(self, mock_next, request_factory, super_user, process_step, transition):
        request = request_factory.get("")
        request.user = super_user
        request.GET = request.GET.copy()
        request.GET["transition_id"] = transition.id
        response = ProcessStepModelViewSet().next(request, pk=process_step.pk)
        assert response.status_code == status.HTTP_200_OK
        assert set(str(x.pk) for x in mock_next.call_args.args) == {
            process_step.pk,
            str(transition.pk),
        }



from django.utils.translation import gettext_lazy as _

from wbcore.contrib.icons import WBIcon
from wbcore.enums import RequestType
from wbcore.metadata.configs import buttons as bt
from wbcore.metadata.configs.buttons.enums import ButtonDefaultColor
from wbcore.metadata.configs.buttons.view_config import ButtonViewConfig
from wbcore.metadata.configs.display.instance_display import create_simple_display

from ...serializers import ChangePasswordSerializer


class UserProfileButtonConfig(ButtonViewConfig):
    CUSTOM_INSTANCE_BUTTONS = {
        bt.WidgetButton(label=_("See Profile"), icon=WBIcon.PERSON.icon, key="see_profile"),
        bt.ActionButton(
            method=RequestType.PATCH,
            identifiers=("authentication:userprofile",),
            action_label=_("Resetting Settings"),
            key="reset_settings",
            description_fields=_("<p>This will reset all widgets on all pages.</p>"),
            icon=WBIcon.UNDO.icon,
            title=_("Reset Settings"),
            label=_("Reset Settings"),
        ),
        bt.ActionButton(
            method=RequestType.PATCH,
            identifiers=("authentication:userprofile",),
            action_label=_("Changing Password"),
            key="change_password",
            description_fields=_("<p>Change Password</p>"),
            serializer=ChangePasswordSerializer,
            icon=WBIcon.EDIT.icon,
            title=_("Change Password"),
            label=_("Change Password"),
            instance_display=create_simple_display([["old_password"], ["new_password"], ["confirm_password"]]),
        ),
    }


class UserModelButtonConfig(ButtonViewConfig):
    CUSTOM_INSTANCE_BUTTONS = CUSTOM_LIST_INSTANCE_BUTTONS = {
        bt.ActionButton(
            method=RequestType.GET,
            identifiers=("authentication:user",),
            action_label=_("Resetting Password"),
            key="reset_password",
            description_fields=_("<p>Reset Password</p>"),
            icon=WBIcon.REGENERATE.icon,
            color=ButtonDefaultColor.WARNING,
            title=_("Reset Password"),
            label=_("Reset Password"),
        ),
        bt.WidgetButton(key="user_activity_chart", label=_("User Activity"), icon=WBIcon.CHART_BARS_HORIZONTAL.icon),
        bt.WidgetButton(key="profile", label=_("Profile"), icon=WBIcon.PERSON.icon),
    }



from wbcore.metadata.configs.endpoints import EndpointViewConfig


class ConditionEndpointConfig(EndpointViewConfig):
    def get_create_endpoint(self, **kwargs) -> str:
        endpoint: str = super().get_create_endpoint(**kwargs)
        if transition_id := self.view.kwargs.get("transition_id"):
            endpoint += f"?transition={transition_id}"
        return endpoint



import json

import pytest
from django.forms.models import model_to_dict
from rest_framework import status
from rest_framework.test import APIRequestFactory

from wbcore.contrib.authentication.factories import SuperUserFactory, UserFactory
from wbcore.contrib.authentication.models import User
from wbcore.contrib.directory.models import ClientManagerRelationship as CMR
from wbcore.test.utils import (
    get_data_from_factory,
    get_kwargs,
    get_model_factory,
)

from ..factories import (
    AddressContactFactory,
    ClientManagerRelationshipFactory,
    CompanyFactory,
    CustomerStatusFactory,
    EntryFactory,
    PersonFactory,
    SocialMediaContactFactory,
)
from ..models import Person
from ..viewsets import (
    AddressContactEntryViewSet,
    BankingContactEntryViewSet,
    ClientManagerViewSet,
    CompanyModelViewSet,
    CompanyRepresentationViewSet,
    CompanyTypeModelViewSet,
    CustomerStatusModelViewSet,
    EmailContactEntryViewSet,
    EntryModelViewSet,
    EntryRepresentationViewSet,
    PersonModelViewSet,
    PersonRepresentationViewSet,
    PositionModelViewSet,
    RelationshipModelViewSet,
    RelationshipTypeModelViewSet,
    SocialMediaContactEntryViewSet,
    SpecializationModelViewSet,
    TelephoneContactEntryViewSet,
    UserIsClientViewSet,
    UserIsManagerViewSet,
    WebsiteContactEntryViewSet,
)


@pytest.fixture
def super_user() -> User:
    return SuperUserFactory()


@pytest.fixture
def normal_user() -> User:
    return UserFactory(is_active=True, is_superuser=False)


@pytest.fixture
def api_request_factory() -> APIRequestFactory:
    return APIRequestFactory()


@pytest.mark.with_db
@pytest.mark.viewset_tests
@pytest.mark.django_db
class TestEntryModelViewSet:
    @pytest.fixture
    def entry_data(self) -> dict[str, object]:
        social_media = SocialMediaContactFactory()
        address = AddressContactFactory()
        data = model_to_dict(EntryFactory.build(), exclude=["profile_image", "signature"])
        data["social_media"] = [social_media.id]
        data["addresses"] = [address.id]
        return data

    @pytest.fixture
    def person_data(self) -> dict[str, object]:
        address = AddressContactFactory()
        data = model_to_dict(PersonFactory.build(), exclude=["profile_image", "signature"])
        data["addresses"] = [address.id]
        return data

    @pytest.fixture
    def company_data(self) -> dict[str, object]:
        address = AddressContactFactory()
        customer_status = CustomerStatusFactory()
        data = model_to_dict(CompanyFactory.build(), exclude=["profile_image", "signature"])
        data["addresses"] = [address.id]
        data["customer_status"] = customer_status.id
        data["employees"] = []
        return data

    @pytest.mark.parametrize(
        "mvs, fixture_name",
        [
            (EntryModelViewSet, "entry_data"),
            (PersonModelViewSet, "person_data"),
            (CompanyModelViewSet, "company_data"),
        ],
    )
    def test_create_entry(
        self,
        request: pytest.FixtureRequest,
        mvs: EntryModelViewSet,
        fixture_name: str,
        api_request_factory: APIRequestFactory,
        super_user: User,
    ):
        # Arrange
        data: dict[str, object] = request.getfixturevalue(fixture_name)
        wsgi_request = api_request_factory.post("", data, format="json")
        wsgi_request.user = super_user
        vs = mvs.as_view({"post": "create"})
        # Act
        response = vs(wsgi_request)
        # Assert
        assert response.status_code == status.HTTP_201_CREATED

    @pytest.mark.parametrize(
        "mvs, factory",
        [
            (EntryModelViewSet, EntryFactory),
            (PersonModelViewSet, PersonFactory),
            (CompanyModelViewSet, CompanyFactory),
        ],
    )
    def test_get_entry(
        self,
        mvs: EntryModelViewSet,
        factory,
        api_request_factory: APIRequestFactory,
        super_user: User,
    ):
        # Arrange
        entry = factory()
        wsgi_request = api_request_factory.get("")
        wsgi_request.user = super_user
        vs = mvs.as_view({"get": "retrieve"})
        # Act
        response = vs(wsgi_request, pk=entry.pk)
        # Assert
        assert response.status_code == status.HTTP_200_OK
        assert not response.data.get("results")
        assert response.data["instance"]["id"] == entry.id

    @pytest.mark.parametrize(
        "mvs, factory",
        [
            (EntryModelViewSet, EntryFactory),
            (PersonModelViewSet, PersonFactory),
            (CompanyModelViewSet, CompanyFactory),
        ],
    )
    def test_get_entry_list(
        self,
        mvs: EntryModelViewSet,
        factory,
        api_request_factory: APIRequestFactory,
        super_user: User,
    ):
        # Arrange
        factory.create_batch(3)
        wsgi_request = api_request_factory.get("")
        wsgi_request.user = super_user
        vs = mvs.as_view({"get": "list"})
        # Act
        response = vs(wsgi_request)
        # Assert
        assert response.status_code == status.HTTP_200_OK
        assert not response.data.get("instance")

    @pytest.mark.parametrize(
        "mvs, factory",
        [
            (EntryModelViewSet, EntryFactory),
            (PersonModelViewSet, PersonFactory),
            (CompanyModelViewSet, CompanyFactory),
        ],
    )
    def test_delete_instance(
        self,
        mvs: EntryModelViewSet,
        factory,
        api_request_factory: APIRequestFactory,
        super_user: User,
    ):
        # Arrange
        model = mvs.get_model()
        entry = factory()
        wsgi_request = api_request_factory.delete("")
        wsgi_request.user = super_user
        vs = mvs.as_view({"delete": "destroy"})
        # Act
        response = vs(wsgi_request, pk=entry.pk)
        # Assert
        assert response.status_code == status.HTTP_204_NO_CONTENT
        assert not model.objects.filter(id=entry.id).exists()

    @pytest.mark.parametrize(
        "mvs, factory, fixture_name",
        [
            (EntryModelViewSet, EntryFactory, "entry_data"),
            (PersonModelViewSet, PersonFactory, "person_data"),
            (CompanyModelViewSet, CompanyFactory, "company_data"),
        ],
    )
    def test_update_instance(
        self,
        request: pytest.FixtureRequest,
        mvs: EntryModelViewSet,
        factory,
        fixture_name: str,
        api_request_factory: APIRequestFactory,
        super_user: User,
    ):
        # Arrange
        entry = factory()
        new_salutation = entry.salutation + "Foo Bar"
        new_data: dict[str, object] = request.getfixturevalue(fixture_name)
        new_data["id"] = entry.id
        # We check for salutation, because it is a field in all 3 tested models
        new_data["salutation"] = new_salutation
        wsgi_request = api_request_factory.put("", data=new_data, format="json")
        wsgi_request.user = super_user
        vs = mvs.as_view({"put": "update"})
        # Act
        response = vs(wsgi_request, pk=entry.pk)
        instance = response.data.get("instance")
        # Assert
        assert response.status_code == status.HTTP_200_OK
        assert instance["id"] == entry.id
        assert instance["salutation"] == new_salutation
        assert instance["salutation"] != entry.salutation

    @pytest.mark.parametrize(
        "mvs, factory",
        [
            (EntryModelViewSet, EntryFactory),
            (PersonModelViewSet, PersonFactory),
            (CompanyModelViewSet, CompanyFactory),
        ],
    )
    def test_patch_instance(
        self,
        mvs: EntryModelViewSet,
        factory,
        api_request_factory: APIRequestFactory,
        super_user: User,
    ):
        # Arrange
        entry = factory()
        wsgi_request = api_request_factory.patch("", data={"salutation": "Foo Bar"})
        wsgi_request.user = super_user
        vs = mvs.as_view({"patch": "partial_update"})
        # Act
        response = vs(wsgi_request, pk=entry.id)
        # Assert
        assert response.status_code == status.HTTP_200_OK
        assert response.data["instance"]["salutation"] == "Foo Bar"

    @pytest.mark.parametrize(
        "viewset, factory",
        [
            (EntryRepresentationViewSet, EntryFactory),
            (PersonRepresentationViewSet, PersonFactory),
            (CompanyRepresentationViewSet, CompanyFactory),
        ],
    )
    def test_get_entry_representation_unauthorized(self, api_request_factory, viewset, factory):
        # Arrange
        entry = factory()
        wsgi_request = api_request_factory.get("")
        wsgi_request.user = UserFactory(is_active=True, is_superuser=False)
        kwargs = get_kwargs(entry, viewset, wsgi_request)
        vs = viewset.as_view({"get": "list"})
        # Act
        response = vs(wsgi_request, **kwargs)
        # Assert
        assert response.status_code == status.HTTP_403_FORBIDDEN


# # =====================================================================================================================
# #                                                  TESTING UTILS VIEWSETS
# # =====================================================================================================================


@pytest.mark.with_db
@pytest.mark.viewset_tests
@pytest.mark.django_db
class TestUtilsViewSets:
    @pytest.mark.parametrize(
        "mvs",
        [
            CustomerStatusModelViewSet,
            PositionModelViewSet,
            CompanyTypeModelViewSet,
            SpecializationModelViewSet,
        ],
    )
    def test_get_utils(self, api_request_factory, super_user, mvs):
        request = api_request_factory.get("")
        request.user = super_user
        factory = get_model_factory(mvs.queryset.model)
        factory.create_batch(3)
        vs = mvs.as_view({"get": "list"})
        response = vs(request)
        assert response.data.get("results")
        assert not response.data.get("instance")
        assert len(response.data.get("results")) == 3
        assert response.status_code == status.HTTP_200_OK

    @pytest.mark.parametrize(
        "mvs",
        [
            CustomerStatusModelViewSet,
            PositionModelViewSet,
            CompanyTypeModelViewSet,
            SpecializationModelViewSet,
        ],
    )
    def test_retrieve_utils(self, api_request_factory, super_user, mvs):
        request = api_request_factory.get("")
        request.user = super_user
        factory = get_model_factory(mvs.queryset.model)
        obj = factory()
        vs = mvs.as_view({"get": "retrieve"})
        response = vs(request, pk=obj.id)
        assert response.data.get("instance")
        assert not response.data.get("results")
        assert response.status_code == status.HTTP_200_OK

    @pytest.mark.parametrize(
        "mvs",
        [
            CustomerStatusModelViewSet,
            PositionModelViewSet,
            CompanyTypeModelViewSet,
            SpecializationModelViewSet,
        ],
    )
    def test_post_utils(self, api_request_factory, super_user, mvs):
        factory = get_model_factory(mvs.queryset.model)
        obj = factory()
        super_user = super_user
        data = get_data_from_factory(obj, mvs, superuser=super_user, delete=True)
        request = api_request_factory.post("", data=data)
        request.user = super_user
        kwargs = get_kwargs(obj, mvs, request)
        vs = mvs.as_view({"post": "create"})
        response = vs(request, **kwargs)
        assert response.status_code == status.HTTP_201_CREATED

    @pytest.mark.parametrize(
        "mvs",
        [
            CustomerStatusModelViewSet,
            PositionModelViewSet,
            CompanyTypeModelViewSet,
            SpecializationModelViewSet,
        ],
    )
    def test_delete_utils(self, api_request_factory, super_user, mvs):
        request = api_request_factory.delete("")
        request.user = super_user
        factory = get_model_factory(mvs.queryset.model)
        obj = factory()
        kwargs = get_kwargs(obj, mvs, request)
        vs = mvs.as_view({"delete": "destroy"})
        response = vs(request, **kwargs, pk=obj.pk)
        assert response.status_code == status.HTTP_204_NO_CONTENT

    @pytest.mark.parametrize(
        "mvs",
        [
            CustomerStatusModelViewSet,
            PositionModelViewSet,
            CompanyTypeModelViewSet,
            SpecializationModelViewSet,
        ],
    )
    def test_put_utils(self, api_request_factory, super_user, mvs):
        factory = get_model_factory(mvs.queryset.model)
        old_obj = factory()
        new_obj = factory()
        user = super_user
        data = get_data_from_factory(new_obj, mvs, superuser=user, delete=True)
        request = api_request_factory.put("", data=data)
        request.user = user
        vs = mvs.as_view({"put": "update"})
        response = vs(request, pk=old_obj.id)
        assert response.status_code == status.HTTP_200_OK
        assert response.data["instance"]["title"] == new_obj.title
        assert not response.data["instance"]["title"] == old_obj.title

    @pytest.mark.parametrize(
        "mvs",
        [
            CustomerStatusModelViewSet,
            PositionModelViewSet,
            CompanyTypeModelViewSet,
            SpecializationModelViewSet,
        ],
    )
    def test_patch_utils(self, api_request_factory, super_user, mvs):
        factory = get_model_factory(mvs.queryset.model)
        obj = factory()
        request = api_request_factory.patch("", data={"title": "New Title"})
        request.user = super_user
        vs = mvs.as_view({"patch": "partial_update"})
        response = vs(request, pk=obj.id)
        assert response.status_code == status.HTTP_200_OK
        assert response.data["instance"]["title"] == "New Title"


# =====================================================================================================================
#                                                  TESTING RELATIONSHIP VIEWSETS
# =====================================================================================================================


@pytest.mark.with_db
@pytest.mark.viewset_tests
@pytest.mark.django_db
class TestRelationshipViewSets:
    @pytest.mark.parametrize("mvs", [RelationshipTypeModelViewSet])
    def test_relationshiptype_retrieve(self, api_request_factory, super_user, mvs):
        request = api_request_factory.get("")
        request.user = super_user
        factory = get_model_factory(mvs.queryset.model)
        type = factory()
        vs = mvs.as_view({"get": "retrieve"})
        response = vs(request, pk=type.id)
        assert response.data.get("instance")
        assert not response.data.get("results")
        assert response.status_code == status.HTTP_200_OK

    @pytest.mark.parametrize("mvs", [RelationshipTypeModelViewSet])
    def test_relationshiptype_get(self, api_request_factory, super_user, mvs):
        request = api_request_factory.get("")
        request.user = super_user
        factory = get_model_factory(mvs.queryset.model)
        factory.create_batch(3)
        vs = mvs.as_view({"get": "list"})
        response = vs(request)
        assert response.data.get("results")
        assert not response.data.get("instance")
        assert len(response.data.get("results")) == 6
        assert response.status_code == status.HTTP_200_OK

    @pytest.mark.parametrize("mvs", [RelationshipTypeModelViewSet])
    def test_relationshiptype_put(self, api_request_factory, super_user, mvs):
        factory = get_model_factory(mvs.queryset.model)
        old_type = factory()
        non_counter_type = factory(counter_relationship=None)
        new_type = factory()
        new_type.counter_relationship = non_counter_type
        user = super_user
        data = get_data_from_factory(new_type, mvs, superuser=user, delete=True)
        request = api_request_factory.put("", data=data)
        request.user = user
        vs = mvs.as_view({"put": "update"})
        response = vs(request, pk=old_type.id)
        assert response.status_code == status.HTTP_200_OK
        assert response.data["instance"]["title"] == new_type.title
        assert not response.data["instance"]["title"] == old_type.title
        assert response.data["instance"]["counter_relationship"] == new_type.counter_relationship.id

    @pytest.mark.parametrize("mvs", [RelationshipTypeModelViewSet])
    def test_relationshiptype_delete(self, api_request_factory, super_user, mvs):
        request = api_request_factory.delete("")
        request.user = super_user
        factory = get_model_factory(mvs.queryset.model)
        type = factory()
        vs = mvs.as_view({"delete": "destroy"})
        response = vs(request, pk=type.id)
        assert response.status_code == status.HTTP_204_NO_CONTENT

    @pytest.mark.parametrize("mvs", [RelationshipTypeModelViewSet])
    def test_relationshiptype_patch(self, api_request_factory, super_user, mvs):
        factory = get_model_factory(mvs.queryset.model)
        type = factory()
        request = api_request_factory.patch("", data={"title": "New Title"})
        request.user = super_user
        vs = mvs.as_view({"patch": "partial_update"})
        response = vs(request, pk=type.id)
        assert response.status_code == status.HTTP_200_OK
        assert response.data["instance"]["title"] == "New Title"

    @pytest.mark.parametrize("mvs", [RelationshipTypeModelViewSet])
    def test_relationshiptype_post(self, api_request_factory, super_user, mvs):
        factory = get_model_factory(mvs.queryset.model)
        type = factory(counter_relationship=None)
        user = super_user
        data = get_data_from_factory(type, mvs, superuser=user, delete=True)
        json_data = json.dumps(data)
        request = api_request_factory.post("", data=json_data, content_type="application/json")
        request.user = user
        kwargs = get_kwargs(type, mvs, request)
        vs = mvs.as_view({"post": "create"})
        response = vs(request, **kwargs)
        assert response.status_code == status.HTTP_201_CREATED

    def test_relationship_list(
        self,
        api_request_factory,
        super_user,
        relationship_factory,
        relationship_type_factory,
    ):
        # Arrange
        relationship_type = relationship_type_factory(counter_relationship=None)
        request = api_request_factory.get("")
        request.user = super_user

        relationship_factory.create_batch(3, relationship_type=relationship_type)
        view = RelationshipModelViewSet.as_view({"get": "list"})
        # Act
        response = view(request).render()
        # Assert
        assert response.status_code == status.HTTP_200_OK
        assert response.data.get("results")
        assert len(response.data["results"]) == 3 * 2

    def test_relationship_retrieve(self, api_request_factory, super_user, relationship_factory):
        # Arrange
        relationship = relationship_factory()
        request = api_request_factory.get("")
        request.user = super_user
        view = RelationshipModelViewSet.as_view({"get": "retrieve"})
        # Act
        response = view(request, pk=relationship.id).render()
        # Assert
        assert response.status_code == status.HTTP_200_OK
        assert json.loads(response.content)["instance"]["from_entry"] == relationship.from_entry.id
        assert json.loads(response.content)["instance"]["to_entry"] == relationship.to_entry.id
        assert json.loads(response.content)["instance"]["relationship_type"] == relationship.relationship_type.id

    def test_relationship_delete(self, api_request_factory, super_user, relationship_factory):
        # Arrange
        relationship = relationship_factory()
        request = api_request_factory.delete("")
        request.user = super_user
        view = RelationshipModelViewSet.as_view({"delete": "destroy"})
        # Act
        response = view(request, pk=relationship.id).render()
        # Assert
        assert response.status_code == status.HTTP_204_NO_CONTENT

    def test_relationship_create(self, api_request_factory, super_user, relationship_factory):
        # Arrange
        relationship = relationship_factory()
        user = super_user
        data = get_data_from_factory(relationship, RelationshipModelViewSet, superuser=user)
        request = api_request_factory.post("", data=data)
        request.user = user
        kwargs = get_kwargs(relationship, RelationshipModelViewSet, request=request, data=data)
        view = RelationshipModelViewSet.as_view({"post": "create"})
        # Act
        response = view(request, kwargs).render()
        # Assert
        assert response.status_code == status.HTTP_201_CREATED

    def test_relationship_update(self, api_request_factory, super_user, relationship_factory):
        # Arrange
        old_relationship = relationship_factory()
        new_relationship = relationship_factory()
        user = super_user
        data = get_data_from_factory(new_relationship, RelationshipModelViewSet, superuser=user)
        request = api_request_factory.put("", data=data)
        request.user = user
        view = RelationshipModelViewSet.as_view({"put": "update"})
        # Act
        response = view(request, pk=old_relationship.id).render()
        # Assert
        assert response.status_code == status.HTTP_200_OK

    def test_relationship_partial_update(self, api_request_factory, super_user, relationship_factory, person_factory):
        # Arrange
        relationship = relationship_factory()
        new_Person = person_factory()
        request = api_request_factory.patch("", data={"to_entry": new_Person.id})
        request.user = super_user
        view = RelationshipModelViewSet.as_view({"patch": "partial_update"})
        # Act
        response = view(request, pk=relationship.id).render()
        # Assert
        assert response.status_code == status.HTTP_200_OK
        assert response.data["instance"]["to_entry"] == new_Person.id


# =====================================================================================================================
#                                                  TESTING CLIENT MANAGER VIEWSETS
# =====================================================================================================================


@pytest.mark.with_db
@pytest.mark.viewset_tests
@pytest.mark.django_db
class TestClientManagerViewSet:
    @pytest.mark.parametrize("mvs", [ClientManagerViewSet])
    def test_None_qs(self, api_request_factory, normal_user, mvs):
        request = api_request_factory.get("")
        request.user = normal_user
        obj = ClientManagerRelationshipFactory()
        kwargs = get_kwargs(obj, mvs, request)
        vs = mvs.as_view({"get": "list"})
        response = vs(request, **kwargs)
        assert response.status_code == status.HTTP_403_FORBIDDEN

    @pytest.mark.parametrize("mvs", [ClientManagerViewSet])
    def test_delete(self, api_request_factory, super_user, mvs):
        request = api_request_factory.delete("")
        request.user = super_user
        obj1 = ClientManagerRelationshipFactory()
        obj2 = ClientManagerRelationshipFactory(client=obj1.client, status=CMR.Status.DRAFT)
        view = mvs.as_view({"delete": "destroy"})
        response = view(request, pk=obj2.id).render()
        assert response.status_code == status.HTTP_204_NO_CONTENT

    @pytest.mark.parametrize("mvs", [ClientManagerViewSet])
    def test_get(self, api_request_factory, super_user, mvs):
        request = api_request_factory.get("")
        request.user = super_user
        ClientManagerRelationshipFactory.create_batch(3)
        vs = mvs.as_view({"get": "list"})
        response = vs(request)
        assert response.data.get("results")
        assert not response.data.get("instance")
        assert len(response.data.get("results")) == 3
        assert response.status_code == status.HTTP_200_OK

    @pytest.mark.parametrize("mvs", [ClientManagerViewSet])
    def test_retrieve(self, api_request_factory, super_user, mvs):
        request = api_request_factory.get("")
        request.user = super_user
        obj = ClientManagerRelationshipFactory()
        vs = mvs.as_view({"get": "retrieve"})
        response = vs(request, pk=obj.pk)
        assert response.data.get("instance")
        assert not response.data.get("results")
        assert response.status_code == status.HTTP_200_OK

    @pytest.mark.parametrize("mvs", [ClientManagerViewSet])
    def test_put(self, api_request_factory, super_user, mvs):
        obj_old = ClientManagerRelationshipFactory(status=CMR.Status.DRAFT)
        obj_new = ClientManagerRelationshipFactory()
        user = super_user
        data = get_data_from_factory(obj_new, mvs, superuser=user, delete=True)
        request = api_request_factory.put("", data=data)
        request.user = user
        kwargs = get_kwargs(obj_old, mvs, request, data)
        vs = mvs.as_view({"put": "update"})
        response = vs(request, pk=obj_old.id, **kwargs).render()
        assert response.status_code == status.HTTP_200_OK
        assert not obj_old.client == obj_new.client
        assert response.data["instance"]["id"] == obj_old.id
        assert response.data["instance"]["client"] == obj_new.client.id
        assert not response.data["instance"]["client"] == obj_old.client.id

    @pytest.mark.parametrize("mvs", [ClientManagerViewSet])
    def test_post(self, api_request_factory, super_user, mvs):
        cmr = ClientManagerRelationshipFactory()
        user = super_user
        data = get_data_from_factory(cmr, mvs, delete=True, superuser=user)
        request = api_request_factory.post("", data=data)
        request.user = user
        kwargs = {}
        view = mvs.as_view({"post": "create"})
        response = view(request, kwargs).render()
        assert response.status_code == status.HTTP_201_CREATED
        assert response.data.get("instance")
        assert response.data["instance"]["client"] == cmr.client.id


@pytest.mark.with_db
@pytest.mark.viewset_tests
@pytest.mark.django_db
class TestUserIsClientViewSet:
    @pytest.mark.parametrize("mvs", [UserIsClientViewSet])
    def test_get(self, api_request_factory, super_user, mvs):
        request = api_request_factory.get("")
        request.user = super_user
        obj = PersonFactory()
        user = Person.get_or_create_with_user(request.user)
        ClientManagerRelationshipFactory(client=user, relationship_manager=obj)
        vs = mvs.as_view({"get": "list"})
        response = vs(request)
        assert response.data.get("results")
        assert not response.data.get("instance")
        assert len(response.data.get("results")) == 1
        assert response.status_code == status.HTTP_200_OK

    @pytest.mark.parametrize("mvs", [UserIsClientViewSet])
    def test_retrieve(self, api_request_factory, super_user, mvs):
        request = api_request_factory.get("")
        request.user = super_user
        user = Person.get_or_create_with_user(request.user)
        rel = ClientManagerRelationshipFactory(client=user)
        vs = mvs.as_view({"get": "retrieve"})
        response = vs(request, pk=rel.pk)
        assert response.data.get("instance")
        assert not response.data.get("results")
        assert response.status_code == status.HTTP_200_OK

    @pytest.mark.parametrize("mvs", [UserIsClientViewSet])
    def test_delete(self, api_request_factory, super_user, mvs):
        request = api_request_factory.delete("")
        request.user = super_user
        obj = PersonFactory()
        user = Person.get_or_create_with_user(request.user)
        ClientManagerRelationshipFactory(client=user, relationship_manager=obj)
        view = mvs.as_view({"delete": "destroy"})
        with pytest.raises(AttributeError):
            view(request, pk=obj.id).render()


@pytest.mark.with_db
@pytest.mark.viewset_tests
@pytest.mark.django_db
class TestUserIsManagerViewSet:
    @pytest.mark.parametrize("mvs", [UserIsManagerViewSet])
    def test_get(self, api_request_factory, super_user, mvs):
        request = api_request_factory.get("")
        request.user = super_user
        obj = EntryFactory()
        user = Person.get_or_create_with_user(request.user)
        ClientManagerRelationshipFactory(client=obj, relationship_manager=user)
        vs = mvs.as_view({"get": "list"})
        response = vs(request)
        assert response.data.get("results")
        assert not response.data.get("instance")
        assert len(response.data.get("results")) == 1
        assert response.status_code == status.HTTP_200_OK

    @pytest.mark.parametrize("mvs", [UserIsManagerViewSet])
    def test_retrieve(self, api_request_factory, super_user, mvs):
        request = api_request_factory.get("")
        request.user = super_user
        obj = EntryFactory()
        user = Person.get_or_create_with_user(request.user)
        ClientManagerRelationshipFactory(client=obj, relationship_manager=user)
        vs = mvs.as_view({"get": "retrieve"})
        response = vs(request, pk=obj.pk)
        assert response.data.get("instance")
        assert not response.data.get("results")
        assert response.status_code == status.HTTP_200_OK

    @pytest.mark.parametrize("mvs", [UserIsManagerViewSet])
    def test_delete(self, api_request_factory, super_user, mvs):
        request = api_request_factory.delete("")
        request.user = super_user
        obj = EntryFactory()
        user = Person.get_or_create_with_user(request.user)
        ClientManagerRelationshipFactory(client=obj, relationship_manager=user)
        view = mvs.as_view({"delete": "destroy"})
        response = view(request, pk=obj.id).render()
        assert response.status_code == status.HTTP_204_NO_CONTENT


@pytest.mark.with_db
@pytest.mark.viewset_tests
@pytest.mark.django_db
class TestContactViewsets:
    @pytest.mark.parametrize(
        "mvs",
        [
            EmailContactEntryViewSet,
            AddressContactEntryViewSet,
            TelephoneContactEntryViewSet,
            WebsiteContactEntryViewSet,
            BankingContactEntryViewSet,
            SocialMediaContactEntryViewSet,
        ],
    )
    def test_get_propagated_contact(self, api_request_factory, super_user, mvs, internal_user_factory):
        request = api_request_factory.get("")
        request.user = super_user
        factory = get_model_factory(mvs.queryset.model)
        obj = factory(entry=internal_user_factory().profile)
        kwargs = get_kwargs(obj, mvs, request)
        vs = mvs.as_view({"get": "list"})
        response = vs(request, **kwargs)
        assert response.status_code == status.HTTP_200_OK
        assert response.data
        assert response.data.get("results")

    @pytest.mark.parametrize(
        "mvs",
        [
            EmailContactEntryViewSet,
            AddressContactEntryViewSet,
            TelephoneContactEntryViewSet,
            WebsiteContactEntryViewSet,
            BankingContactEntryViewSet,
            SocialMediaContactEntryViewSet,
        ],
    )
    def test_post_delete_contact(self, api_request_factory, super_user, mvs):
        factory = get_model_factory(mvs.queryset.model)
        obj = factory()
        user = super_user
        data = get_data_from_factory(obj, mvs, delete=True, superuser=user)
        request = api_request_factory.post("", data)
        request.user = user
        kwargs = get_kwargs(obj, mvs, request=request, data=data)
        vs = mvs.as_view({"post": "create"})
        response = vs(request, **kwargs)
        assert response.status_code == status.HTTP_201_CREATED
        assert response.data.get("instance")

    @pytest.mark.parametrize(
        "mvs",
        [
            EmailContactEntryViewSet,
            AddressContactEntryViewSet,
            TelephoneContactEntryViewSet,
            WebsiteContactEntryViewSet,
            BankingContactEntryViewSet,
            SocialMediaContactEntryViewSet,
        ],
    )
    def test_primary_DeleteEndpointMixin(self, api_request_factory, super_user, mvs):
        request = api_request_factory.delete("")
        request.user = super_user
        factory = get_model_factory(mvs.queryset.model)
        obj = factory()
        obj.primary = False
        obj.save()
        kwargs = get_kwargs(obj, mvs, request)
        vs = mvs.as_view({"delete": "destroy_multiple"})
        response = vs(request=request, **kwargs, pk=obj.pk)
        assert response.status_code == status.HTTP_204_NO_CONTENT



# Generated by Django 5.0.6 on 2024-06-05 14:16

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("directory", "0007_alter_bankingcontact_options"),
    ]

    operations = [
        migrations.AddField(
            model_name="bankingcontact",
            name="access",
            field=models.ManyToManyField(blank=True, related_name="access_to_bank_accounts", to="directory.person"),
        ),
    ]



from django.utils.translation import gettext as _

from wbcore.menus import ItemPermission, MenuItem

USER_ACTIVITY_MENUITEM = MenuItem(
    label=_("User Activity"),
    endpoint="wbcore:authentication:useractivity-list",
    permission=ItemPermission(
        permissions=[
            "authentication.view_user",
            "authentication.add_user",
            "authentication.delete_user",
            "authentication.change_user",
        ]
    ),
)
USER_ACTIVITY_TABLE_MENUITEM = MenuItem(
    label=_("User Activity Table"),
    endpoint="wbcore:authentication:useractivitytable-list",
    permission=ItemPermission(
        permissions=[
            "authentication.view_user",
            "authentication.add_user",
            "authentication.delete_user",
            "authentication.change_user",
        ]
    ),
)
USER_ACTIVITY_CHART_MENUITEM = MenuItem(
    label=_("User Activity Chart"),
    endpoint="wbcore:authentication:useractivitychart-list",
    permission=ItemPermission(
        permissions=[
            "authentication.view_user",
            "authentication.add_user",
            "authentication.delete_user",
            "authentication.change_user",
        ]
    ),
)



from .role import RoleModelSerializer, RoleRepresentationSerializer
from .stadium import StadiumModelSerializer, StadiumRepresentationSerializer
from .person_team import (
    SportPersonModelSerializer,
    SportPersonRepresentationSerializer,
    SportPersonTooltipSerializer,
    PlayerModelSerializer,
    PlayerRepresentationSerializer,
    TreeViewPlayerModelSerializer,
    TeamRepresentationSerializer,
    TeamModelSerializer,
    TeamErrorMessages,
)
from .sport import SportModelSerializer, SportRepresentationSerializer
from .league import LeagueModelSerializer, LeagueRepresentationSerializer, LeagueErrorMessages
from .match_event import (
    MatchModelSerializer,
    ReadOnlyMatchModelSerializer,
    MatchErrorMessages,
    EventErrorMessages,
    MatchRepresentationSerializer,
    EventTypeModelSerializer,
    EventTypeRepresentationSerializer,
    EventModelSerializer,
    EventTypeErrorMessages,
    EventRepresentationSerializer,
    LeaguePlayerStatisticsModelSerializer,
    LeagueTeamStatisticsModelSerializer,
)
from .teamresult import TeamResultsModelSerializer, TeamResultsRepresentationSerializer, ResultErrorMessages
from .season import SeasonModelSerializer, SeasonRepresentationSerializer



# Generated by Django 4.1.8 on 2023-05-23 08:13

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("wbcore", "0011_genericmodel"),
    ]

    operations = [
        migrations.DeleteModel(
            name="Notification",
        ),
    ]



from django.db import models
from django.db.models import CASCADE
from django.utils.translation import gettext
from django.utils.translation import gettext_lazy as _

from wbcore.contrib.icons import WBIcon
from wbcore.contrib.icons.models import IconField
from wbcore.contrib.workflow.models import ProcessStep
from wbcore.models import WBModel


class Transition(WBModel):
    """A transition from one workflow step to the next one."""

    name = models.CharField(max_length=128, verbose_name=_("Name"))
    from_step = models.ForeignKey(
        to="workflow.Step",
        on_delete=CASCADE,
        verbose_name=_("From"),
        related_name="outgoing_transitions",
    )
    to_step = models.ForeignKey(
        to="workflow.Step", on_delete=CASCADE, verbose_name=_("To"), related_name="incoming_transitions"
    )
    icon = IconField(
        max_length=128,
        unique=False,
        verbose_name=_("Icon"),
        blank=False,
        null=False,
        default=WBIcon.START.icon,
    )

    def all_conditions_satisfied(self, process_step: ProcessStep) -> bool:
        if condition_qs := self.associated_conditions.all():
            if instance := process_step.get_instance():
                for cond in condition_qs:
                    if not cond.satisfied(instance):
                        if errors := cond.errors:
                            process_step.step.get_casted_step().set_failed(process_step, errors[0])
                        return False
                return True
            process_step.step.get_casted_step().set_failed(process_step, gettext("No instance or data attached!"))
            return False
        return True

    def __str__(self) -> str:
        return self.name

    @classmethod
    def get_endpoint_basename(cls):
        return "wbcore:workflow:transition"

    @classmethod
    def get_representation_value_key(cls):
        return "id"

    @classmethod
    def get_representation_endpoint(cls):
        return "wbcore:workflow:transitionrepresentation-list"

    @classmethod
    def get_representation_label_key(cls):
        return "{{name}}"

    class Meta:
        verbose_name = _("Transition")
        verbose_name_plural = _("Transitions")
        constraints = [
            models.UniqueConstraint(fields=["name", "to_step"], name="unique_name_to_step"),
        ]
